@@ -38,8 +38,8 @@
 #include "ephy-settings.h"
 #include "ephy-snapshot-service.h"
 #include "ephy-tabs-catalog.h"
-#include "ephy-uri-helpers.h"
 #include "ephy-uri-tester-shared.h"
+#include "ephy-view-source-handler.h"
 #include "ephy-web-app-utils.h"
 #include "ephy-web-extension-proxy.h"
 
@@ -62,12 +62,13 @@ typedef struct {
   EphyDownloadsManager *downloads_manager;
   EphyPermissionsManager *permissions_manager;
   EphyAboutHandler *about_handler;
+  EphyViewSourceHandler *source_handler;
+  char *guid;
   GDBusServer *dbus_server;
   GList *web_extensions;
   EphyFiltersManager *filters_manager;
   EphySearchEngineManager *search_engine_manager;
   GCancellable *cancellable;
-  GList *app_origins;
 } EphyEmbedShellPrivate;
 
 enum {
@@ -162,10 +163,12 @@ ephy_embed_shell_dispose (GObject *object)
   g_clear_object (&priv->global_history_service);
   g_clear_object (&priv->global_gsb_service);
   g_clear_object (&priv->about_handler);
+  g_clear_object (&priv->source_handler);
   g_clear_object (&priv->user_content);
   g_clear_object (&priv->downloads_manager);
   g_clear_object (&priv->permissions_manager);
   g_clear_object (&priv->web_context);
+  g_clear_pointer (&priv->guid, g_free);
   g_clear_object (&priv->dbus_server);
   g_clear_object (&priv->filters_manager);
   g_clear_object (&priv->search_engine_manager);
@@ -173,16 +176,6 @@ ephy_embed_shell_dispose (GObject *object)
   G_OBJECT_CLASS (ephy_embed_shell_parent_class)->dispose (object);
 }
 
-static void
-ephy_embed_shell_finalize (GObject *object)
-{
-  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (EPHY_EMBED_SHELL (object));
-
-  g_list_free_full (priv->app_origins, g_free);
-
-  G_OBJECT_CLASS (ephy_embed_shell_parent_class)->dispose (object);
-}
-
 static void
 web_extension_form_auth_data_message_received_cb (WebKitUserContentManager *manager,
                                                   WebKitJavascriptResult   *message,
@@ -614,6 +607,15 @@ about_request_cb (WebKitURISchemeRequest *request,
   ephy_about_handler_handle_request (priv->about_handler, request);
 }
 
+static void
+source_request_cb (WebKitURISchemeRequest *request,
+                   EphyEmbedShell         *shell)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  ephy_view_source_handler_handle_request (priv->source_handler, request);
+}
+
 static void
 ephy_resource_request_cb (WebKitURISchemeRequest *request)
 {
@@ -715,7 +717,8 @@ initialize_web_extensions (WebKitWebContext *web_context,
 
   private_profile = priv->mode == EPHY_EMBED_SHELL_MODE_PRIVATE || priv->mode == EPHY_EMBED_SHELL_MODE_INCOGNITO;
   browser_mode = priv->mode == EPHY_EMBED_SHELL_MODE_BROWSER;
-  user_data = g_variant_new ("(msssbb)",
+  user_data = g_variant_new ("(smsssbb)",
+                             priv->guid,
                              address,
                              ephy_dot_dir (),
                              ephy_filters_manager_get_adblock_filters_dir (priv->filters_manager),
@@ -779,12 +782,10 @@ ephy_embed_shell_setup_web_extensions_server (EphyEmbedShell *shell)
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   GDBusAuthObserver *observer;
   char *address;
-  char *guid;
   GError *error = NULL;
 
   address = g_strdup_printf ("unix:tmpdir=%s", g_get_tmp_dir ());
 
-  guid = g_dbus_generate_guid ();
   observer = g_dbus_auth_observer_new ();
 
   g_signal_connect (observer, "authorize-authenticated-peer",
@@ -797,7 +798,7 @@ ephy_embed_shell_setup_web_extensions_server (EphyEmbedShell *shell)
    */
   priv->dbus_server = g_dbus_server_new_sync (address,
                                               G_DBUS_SERVER_FLAGS_NONE,
-                                              guid,
+                                              priv->guid,
                                               observer,
                                               NULL,
                                               &error);
@@ -814,7 +815,6 @@ ephy_embed_shell_setup_web_extensions_server (EphyEmbedShell *shell)
 
  out:
   g_free (address);
-  g_free (guid);
   g_object_unref (observer);
 }
 
@@ -982,6 +982,12 @@ ephy_embed_shell_startup (GApplication *application)
   webkit_security_manager_register_uri_scheme_as_local (webkit_web_context_get_security_manager (priv->web_context),
                                                         EPHY_ABOUT_SCHEME);
 
+  /* view source handler */
+  priv->source_handler = ephy_view_source_handler_new ();
+  webkit_web_context_register_uri_scheme (priv->web_context, EPHY_VIEW_SOURCE_SCHEME,
+                                          (WebKitURISchemeRequestCallback)source_request_cb,
+                                          shell, NULL);
+
   /* ephy-resource handler */
   webkit_web_context_register_uri_scheme (priv->web_context, "ephy-resource",
                                           (WebKitURISchemeRequestCallback)ephy_resource_request_cb,
@@ -1077,6 +1083,7 @@ ephy_embed_shell_constructed (GObject *object)
 
   shell = EPHY_EMBED_SHELL (object);
   priv = ephy_embed_shell_get_instance_private (shell);
+  priv->guid = g_dbus_generate_guid ();
   mode = ephy_embed_shell_get_mode (shell);
 
   /* These do not run the EmbedShell application instance, so make sure that
@@ -1103,7 +1110,6 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
   GApplicationClass *application_class = G_APPLICATION_CLASS (klass);
 
   object_class->dispose = ephy_embed_shell_dispose;
-  object_class->finalize = ephy_embed_shell_finalize;
   object_class->set_property = ephy_embed_shell_set_property;
   object_class->get_property = ephy_embed_shell_get_property;
   object_class->constructed = ephy_embed_shell_constructed;
@@ -1394,51 +1400,6 @@ ephy_embed_shell_get_mode (EphyEmbedShell *shell)
   return priv->mode;
 }
 
-/**
- * ephy_embed_shell_add_app_related_uri:
- * @shell: an #EphyEmbedShell
- * @uri: the URI
- **/
-void
-ephy_embed_shell_add_app_related_uri (EphyEmbedShell *shell, const char *uri)
-{
-  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
-  char *origin;
-
-  g_assert (EPHY_IS_EMBED_SHELL (shell));
-  g_assert (priv->mode == EPHY_EMBED_SHELL_MODE_APPLICATION);
-
-  origin = ephy_uri_to_security_origin (uri);
-
-  if (!g_list_find_custom (priv->app_origins, origin, (GCompareFunc)g_strcmp0))
-    priv->app_origins = g_list_append (priv->app_origins, origin);
-}
-
-/**
- * ephy_embed_shell_uri_looks_related_to_application:
- * @shell: an #EphyEmbedShell
- * @uri: the URI
- *
- * Returns: %TRUE if @uri looks related, %FALSE otherwise
- **/
-gboolean
-ephy_embed_shell_uri_looks_related_to_app (EphyEmbedShell *shell,
-                                           const char     *uri)
-{
-  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
-
-  g_assert (EPHY_IS_EMBED_SHELL (shell));
-  g_assert (priv->mode == EPHY_EMBED_SHELL_MODE_APPLICATION);
-
-  for (GList *iter = priv->app_origins; iter != NULL; iter = iter->next) {
-    const char *iter_uri = (const char *)iter->data;
-    if (ephy_embed_utils_urls_have_same_origin (iter_uri, uri))
-      return TRUE;
-  }
-
-  return FALSE;
-}
-
 /**
  * ephy_embed_shell_launch_handler:
  * @shell: an #EphyEmbedShell
@@ -1505,6 +1466,14 @@ ephy_embed_shell_get_user_content_manager (EphyEmbedShell *shell)
   return priv->user_content;
 }
 
+const char *
+ephy_embed_shell_get_guid (EphyEmbedShell *shell)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  return priv->guid;
+}
+
 WebKitWebContext *
 ephy_embed_shell_get_web_context (EphyEmbedShell *shell)
 {
