@@ -102,15 +102,15 @@ ephy_history_record_set_property (GObject      *object,
   switch (prop_id) {
     case PROP_ID:
       g_free (self->id);
-      self->id = g_strdup (g_value_get_string (value));
+      self->id = g_value_dup_string (value);
       break;
     case PROP_TITLE:
       g_free (self->title);
-      self->title = g_strdup (g_value_get_string (value));
+      self->title = g_value_dup_string (value);
       break;
     case PROP_URI:
       g_free (self->uri);
-      self->uri = g_strdup (g_value_get_string (value));
+      self->uri = g_value_dup_string (value);
       break;
     case PROP_VISITS:
       if (self->visits)
@@ -382,7 +382,7 @@ synchronizable_get_id (EphySynchronizable *synchronizable)
   return ephy_history_record_get_id (EPHY_HISTORY_RECORD (synchronizable));
 }
 
-static double
+static gint64
 synchronizable_get_server_time_modified (EphySynchronizable *synchronizable)
 {
   /* No implementation.
@@ -393,7 +393,7 @@ synchronizable_get_server_time_modified (EphySynchronizable *synchronizable)
 
 static void
 synchronizable_set_server_time_modified (EphySynchronizable *synchronizable,
-                                         double              server_time_modified)
+                                         gint64              server_time_modified)
 {
   /* No implementation.
    * We don't care about the server time modified of history records.
